-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt
-- Author: Funky7Monkey

local C = {}
C.__index = C

function C:init()
  self.veloSmoother = newExponentialSmoothing(20, 1)
  self.lastDataPos = vec3()
  self.camLastPos = vec3(0,0,40)
  self.camLastVel = vec3(0,0,0)
  self.camLastDist = vec3(0,0,0)
  self.fov = self.fov or 25

  --- this marks it as base mode. It needs to set position, rotation and fov correctly
  self.baseMode = true
  -- add it to the default camera modes?
  self.register = true
end

function C:update(data)
  local ref  = vec3(data.veh:getNodePosition(self.refNodes.ref))
  local back = vec3(data.veh:getNodePosition(self.refNodes.back))

  local velo = self.veloSmoother:get((self.lastDataPos - data.pos):length())
  self.lastDataPos = data.pos

  local dir = (ref - back):normalized()
  local targetPos = data.pos + vec3(dir.x, dir.y, dir.z) * math.min(25, 10 * velo)
  local camDist = (targetPos - self.camLastPos)
  local camLastVel = self.camLastVel or vec3(0,0,0)
  local camVel = vec3(0,0,0)
  if camLastVel['x'] == 0 then
    if camDist:length() > 100 then
      camVel['x'] = self.camLastVel['x'] + (camDist['x']/6000)
    else
      camVel['x'] = 0
    end
  else
    if camDist:length() > 100 then
      camVel['x'] = self.camLastVel['x'] + (camDist['x']/6000)
    else
      if math.floor(self.camLastVel['x']*5+0.5)/5 ~= 0 then
        camVel['x'] = self.camLastVel['x'] - (camDist['x']/2000)
      else
        camVel['x'] = 0
      end
      if self.camLastDist:length() < camDist:length() then
        if self.camLastDist ~= vec3(0,0,0) then
          camVel['x'] = self.camLastVel['x'] + (camDist['x']/500)
        end
      end
    end
  end
  if camLastVel['y'] == 0 then
    if camDist:length() > 100 then
      camVel['y'] = self.camLastVel['y'] + (camDist['y']/6000)
    else
      camVel['y'] = 0
    end
  else
    if camDist:length() > 100 then
      camVel['y'] = self.camLastVel['y'] + (camDist['y']/6000)
    else
      if math.floor(self.camLastVel['y']*5+0.5)/5 ~= 0 then
        camVel['y'] = self.camLastVel['y'] - (camDist['y']/2000)
      else
        camVel['y'] = 0
      end
      if self.camLastDist:length() < camDist:length() then
        if self.camLastDist ~= vec3(0,0,0) then
          camVel['y'] = self.camLastVel['y'] + (camDist['y']/500)
        end
      end
    end
  end
  self.camLastVel = camVel
  self.camLastDist = camDist
  local camPos = self.camLastPos + camVel
  camPos['z'] = targetPos['z']+40
  self.camLastPos = camPos
  local qdir = quatFromDir((targetPos - camPos):normalized())
  self.fov = 10 + math.min(30, 15 * velo)

  -- set the data, this needs to happen
  data.res.pos = camPos -- required, vec3()
  data.res.rot = qdir -- required, quat()
  data.res.fov = self.fov -- required
  data.res.targetPos = targetPos -- this is optional
  return true
end

-- DO NOT CHANGE CLASS IMPLEMENTATION BELOW

return function(...)
  local o = ... or {}
  setmetatable(o, C)
  o:init()
  return o
end
